﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace ScriptIndicators
{
    public class PI_DonchianChannel : IIndicatorScript
    {
        int length = 20;
        double up_band, low_band, last_up, last_low;

        // Properties
        public string Name
        {
            get { return "pshai's DonchianChannel"; }
        }
        public int TicksBack
        {
            get
            {
                return length;
            }
        }

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> inputField = new List<ScriptParameter>
            {
                new ScriptParameter("Length", ScriptParameterType.Integer, length.ToString()),
            };

            return inputField;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            length = ParseInt(parameters["Length"], length);
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            // TODO: for-loop to add lines

            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "Upper Band", HexLineColor = "#4477FF" },
                new ChartRecord() { ChartIndex = 0, Name = "Lower Band", HexLineColor = "#FF7744" },
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                Convert.ToDecimal(up_band),
                Convert.ToDecimal(low_band)
            };

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
            // In this code example we do not have to do anything
        }

        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                var close = instrument.Close;
                var high = instrument.High;
                var low = instrument.Low;
                var volume = instrument.Volume;
                int position = close.Count() - (1 + context.Offset);
                var result = IndicatorResult.StayNoPosition;

                var highest = high.Skip(position + 1 - length).Take(length).ToArray().Max();
                var lowest = low.Skip(position + 1 - length).Take(length).ToArray().Min();

                last_up = up_band;
                last_low = low_band;

                up_band = highest;
                low_band = lowest;

                // going down
                if (low_band < last_low && up_band <= last_up)
                    result = IndicatorResult.SellShort;

                // going up
                else if (up_band > last_up && low_band >= last_low)
                    result = IndicatorResult.BuyLong;

                return result;
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }
        

        int ParseInt(object value, int lastValue)
        {
            try
            {
                return Convert.ToInt32(value.ToString());
            }
            catch
            {
                return lastValue;
            }
        }
        
    }
}
